@@ -1,118 +1,41 @@
 from __future__ import print_function, division, absolute_import
 
+try:
+    import typing
+except ImportError:
+    import collections as typing
+
 import numpy as np
 import pandas as pd
 import matplotlib
 from matplotlib import pyplot as plt
-from matplotlib.tight_layout import get_renderer
+from matplotlib import colors
+from matplotlib import patches
+
+from .reformat import query, _get_subset_mask
+
+# prevents ImportError on matplotlib versions >3.5.2
+try:
+    from matplotlib.tight_layout import get_renderer
+
+    RENDERER_IMPORTED = True
+except ImportError:
+    RENDERER_IMPORTED = False
 
 
-def _aggregate_data(df, subset_size, sum_over):
-    """
-    Returns
-    -------
-    df : DataFrame
-        full data frame
-    aggregated : Series
-        aggregates
-    """
-    _SUBSET_SIZE_VALUES = ['auto', 'count', 'sum']
-    if subset_size not in _SUBSET_SIZE_VALUES:
-        raise ValueError('subset_size should be one of %s. Got %r'
-                         % (_SUBSET_SIZE_VALUES, subset_size))
-    if df.ndim == 1:
-        # Series
-        input_name = df.name
-        df = pd.DataFrame({'_value': df})
+def _process_data(df, sort_by, sort_categories_by, subset_size,
+                  sum_over, min_subset_size=None, max_subset_size=None,
+                  min_degree=None, max_degree=None, reverse=False):
+    results = query(df, sort_by=sort_by, sort_categories_by=sort_categories_by,
+                    subset_size=subset_size, sum_over=sum_over,
+                    min_subset_size=min_subset_size,
+                    max_subset_size=max_subset_size,
+                    min_degree=min_degree, max_degree=max_degree)
 
-        if subset_size == 'auto' and not df.index.is_unique:
-            raise ValueError('subset_size="auto" cannot be used for a '
-                             'Series with non-unique groups.')
-        if sum_over is not None:
-            raise ValueError('sum_over is not applicable when the input is a '
-                             'Series')
-        if subset_size == 'count':
-            sum_over = False
-        else:
-            sum_over = '_value'
-    else:
-        # DataFrame
-        if sum_over is False:
-            raise ValueError('Unsupported value for sum_over: False')
-        elif subset_size == 'auto' and sum_over is None:
-            sum_over = False
-        elif subset_size == 'count':
-            if sum_over is not None:
-                raise ValueError('sum_over cannot be set if subset_size=%r' %
-                                 subset_size)
-            sum_over = False
-        elif subset_size == 'sum':
-            if sum_over is None:
-                raise ValueError('sum_over should be a field name if '
-                                 'subset_size="sum" and a DataFrame is '
-                                 'provided.')
-
-    gb = df.groupby(level=list(range(df.index.nlevels)))
-    if sum_over is False:
-        aggregated = gb.size()
-        aggregated.name = 'size'
-    elif hasattr(sum_over, 'lower'):
-        aggregated = gb[sum_over].sum()
-    else:
-        raise ValueError('Unsupported value for sum_over: %r' % sum_over)
-
-    if aggregated.name == '_value':
-        aggregated.name = input_name
-
-    return df, aggregated
-
-
-def _check_index(df):
-    # check all indices are boolean
-    if not all(set([True, False]) >= set(level)
-               for level in df.index.levels):
-        raise ValueError('The DataFrame has values in its index that are not '
-                         'boolean')
-    df = df.copy(deep=False)
-    # XXX: this may break if input is not MultiIndex
-    kw = {'levels': [x.astype(bool) for x in df.index.levels],
-          'names': df.index.names,
-          }
-    if hasattr(df.index, 'codes'):
-        # compat for pandas <= 0.20
-        kw['codes'] = df.index.codes
-    else:
-        kw['labels'] = df.index.labels
-    df.index = pd.MultiIndex(**kw)
-    return df
-
-
-def _process_data(df, sort_by, sort_categories_by, subset_size, sum_over):
-    df, agg = _aggregate_data(df, subset_size, sum_over)
+    df = results.data
+    agg = results.subset_sizes
+    totals = results.category_totals
     total = agg.sum()
-    df = _check_index(df)
-
-    totals = [agg[agg.index.get_level_values(name).values.astype(bool)].sum()
-              for name in agg.index.names]
-    totals = pd.Series(totals, index=agg.index.names)
-    if sort_categories_by == 'cardinality':
-        totals.sort_values(ascending=False, inplace=True)
-    elif sort_categories_by is not None:
-        raise ValueError('Unknown sort_categories_by: %r' % sort_categories_by)
-    df = df.reorder_levels(totals.index.values)
-    agg = agg.reorder_levels(totals.index.values)
-
-    if sort_by == 'cardinality':
-        agg = agg.sort_values(ascending=False)
-    elif sort_by == 'degree':
-        gb_degree = agg.groupby(sum, group_keys=False)
-        agg = gb_degree.apply(lambda x: x.sort_index(ascending=False))
-    else:
-        raise ValueError('Unknown sort_by: %r' % sort_by)
-
-    min_value = 0
-    max_value = np.inf
-    agg = agg[np.logical_and(agg >= min_value, agg <= max_value)]
 
     # add '_bin' to df indicating index in agg
     # XXX: ugly!
@@ -127,11 +50,20 @@ def _process_data(df, sort_by, sort_categories_by, subset_size, sum_over):
     df_packed = _pack_binary(df.index.to_frame())
     data_packed = _pack_binary(agg.index.to_frame())
     df['_bin'] = pd.Series(df_packed).map(
-        pd.Series(np.arange(len(data_packed)),
+        pd.Series(np.arange(len(data_packed))[::-1 if reverse else 1],
                   index=data_packed))
+    if reverse:
+        agg = agg[::-1]
+
     return total, df, agg, totals
 
 
+def _multiply_alpha(c, mult):
+    r, g, b, a = colors.to_rgba(c)
+    a *= mult
+    return colors.to_hex((r, g, b, a), keep_alpha=True)
+
+
 class _Transposed:
     """Wrap an object in order to transpose some plotting operations
 
@@ -175,6 +107,12 @@ class _Transposed:
         'set_figheight': 'set_figwidth',
         'set_xlabel': 'set_ylabel',
         'set_ylabel': 'set_xlabel',
+        'set_xlim': 'set_ylim',
+        'set_ylim': 'set_xlim',
+        'get_xlim': 'get_ylim',
+        'get_ylim': 'get_xlim',
+        'set_autoscalex_on': 'set_autoscaley_on',
+        'set_autoscaley_on': 'set_autoscalex_on',
     }
 
 
@@ -203,15 +141,19 @@ class UpSet:
         If a DataFrame, `sum_over` must be a string or False.
     orientation : {'horizontal' (default), 'vertical'}
         If horizontal, intersections are listed from left to right.
-    sort_by : {'cardinality', 'degree'}
+    sort_by : {'cardinality', 'degree', None}
         If 'cardinality', subset are listed from largest to smallest.
         If 'degree', they are listed in order of the number of categories
-        intersected.
+        intersected. If None, the order they appear in the data input is
+        used.
+
+        .. versionchanged:: 0.5
+            Setting None was added.
     sort_categories_by : {'cardinality', None}
         Whether to sort the categories by total cardinality, or leave them
         in the provided order.
 
-        .. versionadded: 0.3
+        .. versionadded:: 0.3
     subset_size : {'auto', 'count', 'sum'}
         Configures how to calculate the size of a subset. Choices are:
 
@@ -230,8 +172,41 @@ class UpSet:
         If `subset_size='sum'` or `'auto'`, then the intersection size is the
         sum of the specified field in the `data` DataFrame. If a Series, only
         None is supported and its value is summed.
-    facecolor : str
-        Color for bar charts and dots.
+    min_subset_size : int, optional
+        Minimum size of a subset to be shown in the plot. All subsets with
+        a size smaller than this threshold will be omitted from plotting.
+        Size may be a sum of values, see `subset_size`.
+
+        .. versionadded:: 0.5
+    max_subset_size : int, optional
+        Maximum size of a subset to be shown in the plot. All subsets with
+        a size greater than this threshold will be omitted from plotting.
+
+        .. versionadded:: 0.5
+    min_degree : int, optional
+        Minimum degree of a subset to be shown in the plot.
+
+        .. versionadded:: 0.5
+    max_degree : int, optional
+        Maximum degree of a subset to be shown in the plot.
+
+        .. versionadded:: 0.5
+    facecolor : 'auto' or matplotlib color or float
+        Color for bar charts and active dots. Defaults to black if
+        axes.facecolor is a light color, otherwise white.
+
+        .. versionchanged:: 0.6
+            Before 0.6, the default was 'black'
+    other_dots_color : matplotlib color or float
+        Color for shading of inactive dots, or opacity (between 0 and 1)
+        applied to facecolor.
+
+        .. versionadded:: 0.6
+    shading_color : matplotlib color or float
+        Color for shading of odd rows in matrix and totals, or opacity (between
+        0 and 1) applied to facecolor.
+
+        .. versionadded:: 0.6
     with_lines : bool
         Whether to show lines joining dots in the matrix, to mark multiple
         categories being intersected.
@@ -241,7 +216,7 @@ class UpSet:
         The intersections plot should be large enough to fit this many matrix
         elements. Set to 0 to disable intersection size bars.
 
-        .. versionchanged: 0.4
+        .. versionchanged:: 0.4
             Setting to 0 is handled.
     totals_plot_elements : int
         The totals plot should be large enough to fit this many matrix
@@ -255,21 +230,34 @@ class UpSet:
         of the intersection relative to the total dataset.
         This may be applied with or without show_counts.
 
-        .. versionadded: 0.4
+        .. versionadded:: 0.4
     """
     _default_figsize = (10, 6)
 
     def __init__(self, data, orientation='horizontal', sort_by='degree',
                  sort_categories_by='cardinality',
                  subset_size='auto', sum_over=None,
-                 facecolor='black',
+                 min_subset_size=None, max_subset_size=None,
+                 min_degree=None, max_degree=None,
+                 facecolor='auto', other_dots_color=.18, shading_color=.05,
                  with_lines=True, element_size=32,
                  intersection_plot_elements=6, totals_plot_elements=2,
                  show_counts='', show_percentages=False):
 
         self._horizontal = orientation == 'horizontal'
         self._reorient = _identity if self._horizontal else _transpose
+        if facecolor == 'auto':
+            bgcolor = matplotlib.rcParams.get('axes.facecolor', 'white')
+            r, g, b, a = colors.to_rgba(bgcolor)
+            lightness = colors.rgb_to_hsv((r, g, b))[-1] * a
+            facecolor = 'black' if lightness >= .5 else 'white'
         self._facecolor = facecolor
+        self._shading_color = (_multiply_alpha(facecolor, shading_color)
+                               if isinstance(shading_color, float)
+                               else shading_color)
+        self._other_dots_color = (_multiply_alpha(facecolor, other_dots_color)
+                                  if isinstance(other_dots_color, float)
+                                  else other_dots_color)
         self._with_lines = with_lines
         self._element_size = element_size
         self._totals_plot_elements = totals_plot_elements
@@ -286,15 +274,205 @@ class UpSet:
                                       sort_by=sort_by,
                                       sort_categories_by=sort_categories_by,
                                       subset_size=subset_size,
-                                      sum_over=sum_over)
-        if not self._horizontal:
-            self.intersections = self.intersections[::-1]
+                                      sum_over=sum_over,
+                                      min_subset_size=min_subset_size,
+                                      max_subset_size=max_subset_size,
+                                      min_degree=min_degree,
+                                      max_degree=max_degree,
+                                      reverse=not self._horizontal)
+        self.subset_styles = [{"facecolor": facecolor}
+                              for i in range(len(self.intersections))]
+        self.subset_legend = []  # pairs of (style, label)
 
     def _swapaxes(self, x, y):
         if self._horizontal:
             return x, y
         return y, x
 
+    def style_subsets(self, present=None, absent=None,
+                      min_subset_size=None, max_subset_size=None,
+                      min_degree=None, max_degree=None,
+                      facecolor=None, edgecolor=None, hatch=None,
+                      linewidth=None, linestyle=None, label=None):
+        """Updates the style of selected subsets' bars and matrix dots
+
+        Parameters are either used to select subsets, or to style them with
+        attributes of :class:`matplotlib.patches.Patch`, apart from label,
+        which adds a legend entry.
+
+        Parameters
+        ----------
+        present : str or list of str, optional
+            Category or categories that must be present in subsets for styling.
+        absent : str or list of str, optional
+            Category or categories that must not be present in subsets for
+            styling.
+        min_subset_size : int, optional
+            Minimum size of a subset to be styled.
+        max_subset_size : int, optional
+            Maximum size of a subset to be styled.
+        min_degree : int, optional
+            Minimum degree of a subset to be styled.
+        max_degree : int, optional
+            Maximum degree of a subset to be styled.
+
+        facecolor : str or matplotlib color, optional
+            Override the default UpSet facecolor for selected subsets.
+        edgecolor : str or matplotlib color, optional
+            Set the edgecolor for bars, dots, and the line between dots.
+        hatch : str, optional
+            Set the hatch. This will apply to intersection size bars, but not
+            to matrix dots.
+        linewidth : int, optional
+            Line width in points for edges.
+        linestyle : str, optional
+            Line style for edges.
+
+        label : str, optional
+            If provided, a legend will be added
+        """
+        style = {"facecolor": facecolor, "edgecolor": edgecolor,
+                 "hatch": hatch,
+                 "linewidth": linewidth, "linestyle": linestyle}
+        style = {k: v for k, v in style.items() if v is not None}
+        mask = _get_subset_mask(self.intersections,
+                                present=present, absent=absent,
+                                min_subset_size=min_subset_size,
+                                max_subset_size=max_subset_size,
+                                min_degree=min_degree, max_degree=max_degree)
+        for idx in np.flatnonzero(mask):
+            self.subset_styles[idx].update(style)
+
+        if label is not None:
+            if "facecolor" not in style:
+                style["facecolor"] = self._facecolor
+            for i, (other_style, other_label) in enumerate(self.subset_legend):
+                if other_style == style:
+                    if other_label != label:
+                        self.subset_legend[i] = (style,
+                                                 other_label + '; ' + label)
+                    break
+            else:
+                self.subset_legend.append((style, label))
+
+    def _plot_bars(self, ax, data, title, colors=None, use_labels=False):
+        ax = self._reorient(ax)
+        ax.set_autoscalex_on(False)
+        data_df = pd.DataFrame(data)
+        if self._horizontal:
+            data_df = data_df.loc[:, ::-1]  # reverse: top row is top of stack
+
+        # TODO: colors should be broadcastable to data_df shape
+        if callable(colors):
+            colors = colors(range(data_df.shape[1]))
+        elif isinstance(colors, (str, type(None))):
+            colors = [colors] * len(data_df)
+
+        if self._horizontal:
+            colors = reversed(colors)
+
+        x = np.arange(len(data_df))
+        cum_y = None
+        all_rects = []
+        for (name, y), color in zip(data_df.items(), colors):
+            rects = ax.bar(x, y, .5, cum_y,
+                           color=color, zorder=10,
+                           label=name if use_labels else None,
+                           align='center')
+            cum_y = y if cum_y is None else cum_y + y
+            all_rects.extend(rects)
+
+        self._label_sizes(ax, rects, 'top' if self._horizontal else 'right')
+
+        ax.xaxis.set_visible(False)
+        for x in ['top', 'bottom', 'right']:
+            ax.spines[self._reorient(x)].set_visible(False)
+
+        tick_axis = ax.yaxis
+        tick_axis.grid(True)
+        ax.set_ylabel(title)
+        return all_rects
+
+    def _plot_stacked_bars(self, ax, by, sum_over, colors, title):
+        df = self._df.set_index("_bin").set_index(by, append=True, drop=False)
+        gb = df.groupby(level=list(range(df.index.nlevels)), sort=True)
+        if sum_over is None and "_value" in df.columns:
+            data = gb["_value"].sum()
+        elif sum_over is None:
+            data = gb.size()
+        else:
+            data = gb[sum_over].sum()
+        data = data.unstack(by).fillna(0)
+        if isinstance(colors, str):
+            colors = matplotlib.cm.get_cmap(colors)
+        elif isinstance(colors, typing.Mapping):
+            colors = data.columns.map(colors).values
+            if pd.isna(colors).any():
+                raise KeyError("Some labels mapped by colors: %r" %
+                               data.columns[pd.isna(colors)].tolist())
+
+        self._plot_bars(ax, data=data, colors=colors, title=title,
+                        use_labels=True)
+
+        handles, labels = ax.get_legend_handles_labels()
+        if self._horizontal:
+            # Make legend order match visual stack order
+            ax.legend(reversed(handles), reversed(labels))
+        else:
+            ax.legend()
+
+    def add_stacked_bars(self, by, sum_over=None, colors=None, elements=3,
+                         title=None):
+        """Add a stacked bar chart over subsets when :func:`plot` is called.
+
+        Used to plot categorical variable distributions within each subset.
+
+        .. versionadded:: 0.6
+
+        Parameters
+        ----------
+        by : str
+            Column name within the dataframe for color coding the stacked bars,
+            containing discrete or categorical values.
+        sum_over : str, optional
+            Ordinarily the bars will chart the size of each group. sum_over
+            may specify a column which will be summed to determine the size
+            of each bar.
+        colors : Mapping, list-like, str or callable, optional
+            The facecolors to use for bars corresponding to each discrete
+            label, specified as one of:
+
+            Mapping
+                Maps from label to matplotlib-compatible color specification.
+            list-like
+                A list of matplotlib colors to apply to labels in order.
+            str
+                The name of a matplotlib colormap name.
+            callable
+                When called with the number of labels, this should return a
+                list-like of that many colors.  Matplotlib colormaps satisfy
+                this callable API.
+            None
+                Uses the matplotlib default colormap.
+        elements : int, default=3
+            Size of the axes counted in number of matrix elements.
+        title : str, optional
+            The axis title labelling bar length.
+
+        Returns
+        -------
+        None
+        """
+        # TODO: allow sort_by = {"lexical", "sum_squares", "rev_sum_squares",
+        #                        list of labels}
+        self._subset_plots.append({'type': 'stacked_bars',
+                                   'by': by,
+                                   'sum_over': sum_over,
+                                   'colors': colors,
+                                   'title': title,
+                                   'id': 'extra%d' % len(self._subset_plots),
+                                   'elements': elements})
+
     def add_catplot(self, kind, value=None, elements=3, **kw):
         """Add a seaborn catplot over subsets when :func:`plot` is called.
 
@@ -332,12 +510,16 @@ class UpSet:
                                    'elements': elements,
                                    'kw': kw})
 
-    def _plot_catplot(self, ax, value, kind, kw):
-        df = self._df
-        if value is None and '_value' in df.columns:
+    def _check_value(self, value):
+        if value is None and '_value' in self._df.columns:
             value = '_value'
         elif value is None:
             raise ValueError('value can only be None when data is a Series')
+        return value
+
+    def _plot_catplot(self, ax, value, kind, kw):
+        df = self._df
+        value = self._check_value(value)
         kw = kw.copy()
         if self._horizontal:
             kw['orient'] = 'v'
@@ -372,32 +554,40 @@ class UpSet:
             fig = plt.gcf()
 
         # Determine text size to determine figure size / spacing
-        r = get_renderer(fig)
-        t = fig.text(0, 0, '\n'.join(self.totals.index.values))
-        textw = t.get_window_extent(renderer=r).width
+        text_kw = {"size": matplotlib.rcParams['xtick.labelsize']}
+        # adding "x" ensures a margin
+        t = fig.text(0, 0, '\n'.join(str(label) + "x"
+                                     for label in self.totals.index.values),
+                     **text_kw)
+        window_extent_args = {}
+        if RENDERER_IMPORTED:
+            window_extent_args["renderer"] = get_renderer(fig)
+        textw = t.get_window_extent(**window_extent_args).width
         t.remove()
 
-        MAGIC_MARGIN = 10  # FIXME
-        figw = self._reorient(fig.get_window_extent(renderer=r)).width
+        window_extent_args = {}
+        if RENDERER_IMPORTED:
+            window_extent_args["renderer"] = get_renderer(fig)
+        figw = self._reorient(
+            fig.get_window_extent(**window_extent_args)).width
 
         sizes = np.asarray([p['elements'] for p in self._subset_plots])
+        fig = self._reorient(fig)
 
+        non_text_nelems = len(self.intersections) + self._totals_plot_elements
         if self._element_size is None:
-            colw = (figw - textw - MAGIC_MARGIN) / (len(self.intersections) +
-                                                    self._totals_plot_elements)
+            colw = (figw - textw) / non_text_nelems
         else:
-            fig = self._reorient(fig)
             render_ratio = figw / fig.get_figwidth()
             colw = self._element_size / 72 * render_ratio
-            figw = (colw * (len(self.intersections) +
-                            self._totals_plot_elements) +
-                    MAGIC_MARGIN + textw)
+            figw = colw * (non_text_nelems + np.ceil(textw / colw) + 1)
             fig.set_figwidth(figw / render_ratio)
             fig.set_figheight((colw * (n_cats + sizes.sum())) /
                               render_ratio)
 
-        text_nelems = int(np.ceil(figw / colw - (len(self.intersections) +
-                                                 self._totals_plot_elements)))
+        text_nelems = int(np.ceil(figw / colw - non_text_nelems))
+        # print('textw', textw, 'figw', figw, 'colw', colw,
+        #       'ncols', figw/colw, 'text_nelems', text_nelems)
 
         GS = self._reorient(matplotlib.gridspec.GridSpec)
         gridspec = GS(*self._swapaxes(n_cats + (sizes.sum() or 0),
@@ -432,26 +622,60 @@ class UpSet:
         data = self.intersections
         n_cats = data.index.nlevels
 
-        idx = np.flatnonzero(data.index.to_frame()[data.index.names].values)
-        c = np.array(['lightgrey'] * len(data) * n_cats, dtype='O')
-        c[idx] = self._facecolor
+        inclusion = data.index.to_frame().values
+
+        # Prepare styling
+        styles = [
+            [
+                self.subset_styles[i]
+                if inclusion[i, j]
+                else {"facecolor": self._other_dots_color, "linewidth": 0}
+                for j in range(n_cats)
+            ]
+            for i in range(len(data))
+        ]
+        styles = sum(styles, [])  # flatten nested list
+        style_columns = {"facecolor": "facecolors",
+                         "edgecolor": "edgecolors",
+                         "linewidth": "linewidths",
+                         "linestyle": "linestyles",
+                         "hatch": "hatch"}
+        styles = pd.DataFrame(styles).reindex(columns=style_columns.keys())
+        styles["linewidth"].fillna(1, inplace=True)
+        styles["facecolor"].fillna(self._facecolor, inplace=True)
+        styles["edgecolor"].fillna(styles["facecolor"], inplace=True)
+        styles["linestyle"].fillna("solid", inplace=True)
+        del styles["hatch"]  # not supported in matrix (currently)
+
         x = np.repeat(np.arange(len(data)), n_cats)
         y = np.tile(np.arange(n_cats), len(data))
+
+        # Plot dots
         if self._element_size is not None:
             s = (self._element_size * .35) ** 2
         else:
             # TODO: make s relative to colw
             s = 200
-        ax.scatter(*self._swapaxes(x, y), c=c.tolist(), linewidth=0, s=s)
+        ax.scatter(*self._swapaxes(x, y), s=s, zorder=10,
+                   **styles.rename(columns=style_columns))
 
+        # Plot lines
         if self._with_lines:
+            idx = np.flatnonzero(inclusion)
             line_data = (pd.Series(y[idx], index=x[idx])
                          .groupby(level=0)
                          .aggregate(['min', 'max']))
+            colors = pd.Series([
+                style.get("edgecolor", style.get("facecolor", self._facecolor))
+                for style in self.subset_styles],
+                name="color")
+            line_data = line_data.join(colors)
             ax.vlines(line_data.index.values,
                       line_data['min'], line_data['max'],
-                      lw=2, colors=self._facecolor)
+                      lw=2, colors=line_data["color"],
+                      zorder=5)
 
+        # Ticks and axes
         tick_axis = ax.yaxis
         tick_axis.set_ticks(np.arange(n_cats))
         tick_axis.set_ticklabels(data.index.names,
@@ -461,25 +685,26 @@ class UpSet:
         if not self._horizontal:
             ax.yaxis.set_ticks_position('top')
         ax.set_frame_on(False)
-        ax.set_xlim(xmin=.5, xmax=x[-1] + .5, auto=False)
+        ax.set_xlim(-.5, x[-1] + .5, auto=False)
+        ax.grid(False)
 
     def plot_intersections(self, ax):
         """Plot bars indicating intersection size
         """
-        ax = self._reorient(ax)
-        ax.set_autoscalex_on(False)
-        rects = ax.bar(np.arange(len(self.intersections)), self.intersections,
-                       .5, color=self._facecolor, zorder=10, align='center')
+        rects = self._plot_bars(ax, self.intersections,
+                                title='Intersection size',
+                                colors=self._facecolor)
+        for style, rect in zip(self.subset_styles, rects):
+            style = style.copy()
+            style.setdefault("edgecolor",
+                             style.get("facecolor", self._facecolor))
+            for attr, val in style.items():
+                getattr(rect, "set_" + attr)(val)
 
-        self._label_sizes(ax, rects, 'top' if self._horizontal else 'right')
-
-        ax.xaxis.set_visible(False)
-        for x in ['top', 'bottom', 'right']:
-            ax.spines[self._reorient(x)].set_visible(False)
-
-        tick_axis = ax.yaxis
-        tick_axis.grid(True)
-        ax.set_ylabel('Intersection size')
+        if self.subset_legend:
+            styles, labels = zip(*self.subset_legend)
+            styles = [patches.Patch(**patch_style) for patch_style in styles]
+            ax.legend(styles, labels)
 
     def _label_sizes(self, ax, rects, where):
         if not self._show_counts and not self._show_percentages:
@@ -515,7 +740,7 @@ class UpSet:
         if where == 'right':
             margin = 0.01 * abs(np.diff(ax.get_xlim()))
             for rect in rects:
-                width = rect.get_width()
+                width = rect.get_width() + rect.get_x()
                 ax.text(width + margin,
                         rect.get_y() + rect.get_height() * .5,
                         fmt % make_args(width),
@@ -523,7 +748,7 @@ class UpSet:
         elif where == 'left':
             margin = 0.01 * abs(np.diff(ax.get_xlim()))
             for rect in rects:
-                width = rect.get_width()
+                width = rect.get_width() + rect.get_x()
                 ax.text(width + margin,
                         rect.get_y() + rect.get_height() * .5,
                         fmt % make_args(width),
@@ -531,7 +756,7 @@ class UpSet:
         elif where == 'top':
             margin = 0.01 * abs(np.diff(ax.get_ylim()))
             for rect in rects:
-                height = rect.get_height()
+                height = rect.get_height() + rect.get_y()
                 ax.text(rect.get_x() + rect.get_width() * .5,
                         height + margin,
                         fmt % make_args(height),
@@ -555,6 +780,7 @@ class UpSet:
             ax.spines[self._reorient(x)].set_visible(False)
         ax.yaxis.set_visible(False)
         ax.xaxis.grid(True)
+        ax.yaxis.grid(False)
         ax.patch.set_visible(False)
 
     def plot_shading(self, ax):
@@ -562,7 +788,7 @@ class UpSet:
         for i in range(0, len(self.totals), 2):
             rect = plt.Rectangle(self._swapaxes(0, i - .4),
                                  *self._swapaxes(*(1, .8)),
-                                 facecolor='#f5f5f5', lw=0, zorder=0)
+                                 facecolor=self._shading_color, lw=0, zorder=0)
             ax.add_patch(rect)
         ax.set_frame_on(False)
         ax.tick_params(
@@ -574,6 +800,7 @@ class UpSet:
             top=False,
             labelbottom=False,
             labelleft=False)
+        ax.grid(False)
         ax.set_xticks([])
         ax.set_yticks([])
         ax.set_xticklabels([])
@@ -612,13 +839,22 @@ class UpSet:
                                                  sharex=matrix_ax)
             if plot['type'] == 'default':
                 self.plot_intersections(ax)
-            elif plot['type'] == 'catplot':
-                self._plot_catplot(ax, plot['value'], plot['kind'], plot['kw'])
+            elif plot['type'] in self.PLOT_TYPES:
+                kw = plot.copy()
+                del kw['type']
+                del kw['elements']
+                del kw['id']
+                self.PLOT_TYPES[plot['type']](self, ax, **kw)
             else:
                 raise ValueError('Unknown subset plot type: %r' % plot['type'])
             out[plot['id']] = ax
         return out
 
+    PLOT_TYPES = {
+        'catplot': _plot_catplot,
+        'stacked_bars': _plot_stacked_bars,
+    }
+
     def _repr_html_(self):
         fig = plt.figure(figsize=self._default_figsize)
         self.plot(fig=fig)
